#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = void 0;
const meow_1 = __importDefault(require("meow"));
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const chokidar_1 = __importDefault(require("chokidar"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const slash_1 = __importDefault(require("slash"));
const ejs_1 = __importDefault(require("ejs"));
const template_1 = require("./template");
let readConfig;
try {
    readConfig = require('@remix-run/dev/config').readConfig;
}
catch (e) {
    try {
        readConfig = require('@remix-run/dev/dist/config').readConfig;
    }
    catch (e) {
        readConfig = require('@vercel/remix-run-dev/dist/config').readConfig;
    }
}
const helpText = `
Usage
$ remix-routes

Options
--watch, -w  Watch for routes changes
--strict, -s  Enable strict mode
--outputDirPath, -o Specify the output path for "remix-routes.d.ts". Defaults to "./node_modules" if arg is not given.
`;
const DEFAULT_OUTPUT_DIR_PATH = './node_modules';
const cli = (0, meow_1.default)(helpText, {
    flags: {
        watch: {
            type: 'boolean',
            alias: 'w',
        },
        strict: {
            type: 'boolean',
            alias: 's',
        },
        outputDirPath: {
            type: 'string',
            alias: 'o',
            default: DEFAULT_OUTPUT_DIR_PATH,
        }
    },
});
async function buildHelpers(config) {
    const routesInfo = {};
    const routeIds = [];
    const handleRoutesRecursive = (parentId, parentPath = []) => {
        let routes = Object.values(config.routes).filter((route) => route.parentId === parentId);
        routes.forEach((route) => {
            let currentPath = parentPath;
            routeIds.push(route.id);
            if (route.id === 'root') {
                routesInfo['/'] = {
                    fileName: route.file,
                    params: [],
                };
            }
            else {
                currentPath = [...currentPath, route];
                const fullPath = dedupPrefixSlash(currentPath.reduce((acc, curr) => [acc, trimSuffixSlash(curr.path)].filter(p => p != undefined).join('/'), ''));
                const paramsNames = parse(currentPath);
                routesInfo[fullPath] = {
                    fileName: route.file,
                    params: paramsNames
                };
            }
            handleRoutesRecursive(route.id, currentPath);
        });
    };
    handleRoutesRecursive();
    return [routesInfo, routeIds];
}
async function build(remixRoot, flags) {
    const config = await readConfig(remixRoot);
    const [routesInfo, routeIds] = await buildHelpers(config);
    generate(config, routesInfo, routeIds, remixRoot, flags);
}
exports.build = build;
async function watch(remixRoot, flags) {
    build(remixRoot, flags);
    const { appDirectory } = await readConfig(remixRoot);
    chokidar_1.default
        .watch([
        path.join(appDirectory, 'routes/**/*.{ts,tsx}'),
        path.join(remixRoot, 'remix.config.js'),
    ])
        .on('change', () => {
        build(remixRoot, flags);
    });
    console.log('Watching for routes changes...');
}
function generate(config, routesInfo, routeIds, remixRoot, flags) {
    const outputPath = path.join(remixRoot, flags.outputDirPath);
    const relativeAppDirPath = (0, slash_1.default)(path.relative(outputPath, config.appDirectory));
    routeIds.sort((a, b) => a.localeCompare(b));
    const tsCode = ejs_1.default.render(template_1.template, {
        strictMode: flags.strict,
        relativeAppDirPath,
        routes: Object.entries(routesInfo).map(([route, { fileName, params }]) => ({
            route,
            params,
            fileName: (0, slash_1.default)(fileName.replace(/\.tsx?$/, '')),
        })).sort((a, b) => a.route.localeCompare(b.route)),
        routeIds,
    });
    if (!fs.existsSync(outputPath)) {
        mkdirp_1.default.sync(outputPath);
    }
    fs.writeFileSync(path.join(outputPath, 'remix-routes.d.ts'), tsCode);
}
function parse(routes) {
    const paramNames = [];
    routes.forEach((route) => {
        return (route.path &&
            paramNames.push(...route.path
                .split('/')
                .filter((seg) => seg.startsWith(':') || seg == '*')
                .map((param) => param.replace(':', '').replace('*', '"*"'))));
    });
    return paramNames;
}
if (require.main === module) {
    (async function () {
        var _a;
        const remixRoot = (_a = process.env.REMIX_ROOT) !== null && _a !== void 0 ? _a : process.cwd();
        console.log(cli.flags);
        if (cli.flags.watch) {
            watch(remixRoot, cli.flags);
        }
        else {
            build(remixRoot, cli.flags);
        }
    })();
}
function dedupPrefixSlash(path) {
    return path.replace(/^\/+/, '/');
}
function trimSuffixSlash(path) {
    if (!path)
        return path;
    return path.replace(/\/+$/, '');
}
